#include <stdio.h>
#include <stdlib.h>

struct int_node {
  int val;
  struct int_node * prev; /* NULL => no previous elements */
  struct int_node * next; /* NULL => empty */
};

typedef struct int_node * node;

node make_node(node prev_ls, int i, node next_ls) {
  node ls2;
  
  ls2 = (node)malloc(sizeof(struct int_node));
  
  ls2->val = i;
  ls2->prev = prev_ls;
  ls2->next = next_ls;

  if (next_ls != NULL)
    next_ls->prev = ls2;
  if (prev_ls != NULL)
    prev_ls->next = ls2;
  
  return ls2;
}

/* We can treat a `node' as starting a list going forward: */
void print_list(struct int_node * ls) {
  for (; ls != NULL; ls = ls->next) {
    printf("%d\n", ls->val);
  }
}

/* We can treat a `node' as starting a list going backward: */
void print_list_backwards(struct int_node * ls) {
  for (; ls != NULL; ls = ls->prev) {
    printf("%d\n", ls->val);
  }
}

/* ---------------------------------------- */

struct container {
  node hd;
  node tl;
};

typedef struct container * list_container;

list_container make_list_container() {
  list_container lc;

  lc = (list_container)malloc(sizeof(struct container));
  lc->hd = NULL;
  lc->tl = NULL;

  return lc;
}

void add_to_front(list_container lc, int i) {
  lc->hd = make_node(NULL, i, lc->hd);
  if (lc->tl == NULL)
    lc->tl = lc->hd;
}

void add_to_back(list_container lc, int i) {
  lc->tl = make_node(lc->tl, i, NULL);
  if (lc->hd == NULL)
    lc->hd = lc->tl;
}

node get_list(list_container lc) {
  return lc->hd;
}

node get_list_end(list_container lc) {
  return lc->tl;
}

/* ---------------------------------------- */

/* Tests */

void failed() {
  printf("test failed\n");
  exit(1);
}

void check_ptr(void *a, void *b) {
  if (a != b) failed();
}

void check_int(int a, int b) {
  if (a != b) failed();
}

void test() {
  list_container lc;

  lc = make_list_container();
  check_ptr(get_list(lc), NULL);
  add_to_front(lc, 5); /* start with front */
  check_int(get_list(lc)->val, 5);
  check_ptr(get_list(lc)->next, NULL);
  add_to_back(lc, 6);
  check_int(get_list(lc)->val, 5);
  check_int(get_list(lc)->next->val, 6);
  check_ptr(get_list(lc)->next->next, NULL);
  add_to_back(lc, 7);
  add_to_front(lc, 4);
  check_int(get_list(lc)->val, 4);
  check_int(get_list(lc)->next->val, 5);
  check_int(get_list(lc)->next->next->val, 6);
  check_int(get_list(lc)->next->next->next->val, 7);
  check_ptr(get_list(lc)->next->next->next->next, NULL);

  check_int(get_list_end(lc)->val, 7);
  check_int(get_list_end(lc)->prev->val, 6);
  check_int(get_list_end(lc)->prev->prev->val, 5);
  check_int(get_list_end(lc)->prev->prev->prev->val, 4);
  check_ptr(get_list_end(lc)->prev->prev->prev->prev, NULL);

  lc = make_list_container();
  check_ptr(get_list(lc), NULL);
  add_to_back(lc, 5); /* start with back */
  check_int(get_list(lc)->val, 5);
  check_ptr(get_list(lc)->next, NULL);
  add_to_front(lc, 6);
  check_int(get_list(lc)->val, 6);
  check_int(get_list(lc)->next->val, 5);
  check_ptr(get_list(lc)->next->next, NULL);

  check_int(get_list_end(lc)->val, 5);
  check_int(get_list_end(lc)->prev->val, 6);
  check_ptr(get_list_end(lc)->prev->prev, NULL);
}

/* ---------------------------------------- */

int main() {
  list_container lc;
  char buffer[256];

  test();

  lc = make_list_container();
  
  for (; fgets(buffer, 256, stdin); ) {
    add_to_back(lc, atoi(buffer));
  }

  print_list_backwards(get_list_end(lc));
  
  return 0;
}
