#include <stdio.h>
#include <stdlib.h>

struct int_cons {
  int first;
  struct int_cons * rest; /* NULL => empty */
};

typedef struct int_cons * list;

list cons(int i, list ls) {
  list ls2;
  
  ls2 = (list)malloc(sizeof(struct int_cons));
  
  ls2->first = i;
  ls2->rest = ls;
  
  return ls2;
}

void print_list(struct int_cons * ls) {
  for (; ls != NULL; ls = ls->rest) {
    printf("%d\n", ls->first);
  }
}

/* ---------------------------------------- */

struct container {
  list hd;
  list tl;
};

typedef struct container * list_container;

list_container make_list_container() {
  list_container lc;

  lc = (list_container)malloc(sizeof(struct container));
  lc->hd = NULL;
  lc->tl = NULL;

  return lc;
}

void add_to_front(list_container lc, int i) {
  lc->hd = cons(i, lc->hd);
  if (lc->tl == NULL)
    lc->tl = lc->hd;
}

void add_to_back(list_container lc, int i) {
  if (lc->tl == NULL) {
    lc->hd = cons(i, NULL);
    lc->tl = lc->hd;
  } else {
    lc->tl->rest = cons(i, NULL);
    lc->tl = lc->tl->rest;
  }
}

list get_list(list_container lc) {
  return lc->hd;
}

/* ---------------------------------------- */

/* Tests */

void failed() {
  printf("test failed\n");
  exit(1);
}

void check_ptr(void *a, void *b) {
  if (a != b) failed();
}

void check_int(int a, int b) {
  if (a != b) failed();
}

void test() {
  list_container lc;

  lc = make_list_container();
  check_ptr(get_list(lc), NULL);
  add_to_front(lc, 5); /* start with front */
  check_int(get_list(lc)->first, 5);
  check_ptr(get_list(lc)->rest, NULL);
  add_to_back(lc, 6);
  check_int(get_list(lc)->first, 5);
  check_int(get_list(lc)->rest->first, 6);
  check_ptr(get_list(lc)->rest->rest, NULL);
  add_to_back(lc, 7);
  add_to_front(lc, 4);
  check_int(get_list(lc)->first, 4);
  check_int(get_list(lc)->rest->first, 5);
  check_int(get_list(lc)->rest->rest->first, 6);
  check_int(get_list(lc)->rest->rest->rest->first, 7);
  check_ptr(get_list(lc)->rest->rest->rest->rest, NULL);

  lc = make_list_container();
  check_ptr(get_list(lc), NULL);
  add_to_back(lc, 5); /* start with back */
  check_int(get_list(lc)->first, 5);
  check_ptr(get_list(lc)->rest, NULL);
  add_to_front(lc, 6);
  check_int(get_list(lc)->first, 6);
  check_int(get_list(lc)->rest->first, 5);
  check_ptr(get_list(lc)->rest->rest, NULL);
}

/* ---------------------------------------- */

int main() {
  list_container lc;
  char buffer[256];

  test();

  lc = make_list_container();
  
  for (; fgets(buffer, 256, stdin); ) {
    add_to_back(lc, atoi(buffer));
  }

  print_list(get_list(lc));
  
  return 0;
}
